@@ -1,22 +1,36 @@
-from .models import CustomerExtension, CustomerExtensionExtended, Template
+from abc import abstractmethod
+from dataclasses import dataclass
+from .models import CustomerExtension, CustomerExtensionExtended, ValidatedTemplate
 from .models import Departure
 from .models import Route
 from .models import Selection
-from typing import List
+from typing import Iterator, Optional
 from datetime import datetime
 from .methods import calc_departures, calc_route_departure, recalculate_lead_time
+from typing import Protocol
+
+def tostr(string: Optional[str]) -> str:
+    if not string:
+        return ''
+    else:
+        return string
 
 
-def MakeRoute(data: Template) -> Route:
-    route = Route(
+
+def MakeRoute(data: ValidatedTemplate) -> Iterator[Route]:
+    data = data.copy()
+
+    tostr = lambda x: str(x) if x != None else ''
+
+    yield Route.construct(
         ROUT=data.ROUT,
         RUTP=6,
-        TX40=data.EDEL\
-            + '_' + data.EDEU\
-            + '_' + data.MODL,
-        TX15=data.EDEL\
-            + '_' + data.EDEU\
-            + '_' + data.MODL,
+        TX40=tostr(data.EDEL)\
+            + '_' + tostr(data.EDEU)\
+            + '_' + tostr(data.MODL),
+        TX15=tostr(data.EDEL)\
+            + '_' + tostr(data.EDEU)\
+            + '_' + tostr(data.MODL),
         RESP=data.RRSP,
         SDES=data.EDEL,
         DLMC=1,
@@ -24,22 +38,21 @@ def MakeRoute(data: Template) -> Route:
         TSID=data.EDEU
     )
 
-    return route
 
 
-def MakeDeparture(data: Template) -> List[Departure]:
+def MakeDeparture(data: ValidatedTemplate) -> Iterator[Departure]:
     data = data.copy()
+
     list_of_departure_days = [data.DDOW]\
         if not data.ADOW\
         else calc_departures(data.DDOW, data.ARDY)
      
-    departures = []
-    for departureDays in list_of_departure_days:
-        RODN = calc_route_departure(departureDays, data.ARDY) if not data.RODN else data.RODN
-        ARDY = recalculate_lead_time(departureDays, data.ARDY) if data.ADOW else data.ARDY
-        ARDY = int(data.ARDX) if data.ARDX else data.ARDY
+    for DDOW in list_of_departure_days:
+        RODN = calc_route_departure(DDOW, data.ARDY) if not data.RODN else data.RODN
+        ARDY = recalculate_lead_time(DDOW, data.ARDY) if data.ADOW else data.ARDY
+        ARDY = int(data.ARDX) if data.ARDX else ARDY
     
-        departure =  Departure(
+        yield Departure.construct(
             WWROUT = data.ROUT,
             WWRODN = RODN,
             WRRESP = data.DRSP,
@@ -55,7 +68,7 @@ def MakeDeparture(data: Template) -> List[Departure]:
             WEFWLD = data.FWLD,
             WEFWLH = data.FWLH,
             WEFWLM = data.FWLM,
-            WRDDOW = data.DDOW,
+            WRDDOW = DDOW,
             WRDETH = data.DETH,
             WRDETM = data.DETM,
             WRVFDT = datetime.now().strftime('%y%m%d'),
@@ -63,13 +76,11 @@ def MakeDeparture(data: Template) -> List[Departure]:
             WRARHH = data.ARHH,
             WRARMM = data.ARMM
         )
-        departures.append(departure)
-
-    return departures
 
 
-def MakeSelection(data: Template) -> Selection:
-    selection = Selection(
+def MakeSelection(data: ValidatedTemplate) -> Iterator[Selection]:
+    data = data.copy()
+    yield Selection.construct(
         EDES = data.EDEL,
         PREX = ' 6',  # with preceeding space
         OBV1 = data.EDEU,
@@ -85,44 +96,41 @@ def MakeSelection(data: Template) -> Selection:
             else None
     )
 
-    return selection
 
 
-def MakeCustomerExtension(data: Template) -> List[CustomerExtension]:
-    list_of_customer_extensions = []
+def MakeCustomerExtension(data: ValidatedTemplate) -> Iterator[CustomerExtension]:
+    data = data.copy()
+
+    list_of_departure_days = [data.DDOW]\
+    if not data.ADOW\
+    else calc_departures(data.DDOW, data.ARDY)
 
     if data.PCUD or data.PCUH or data.PCUM:
-        list_of_customer_extensions.append(
-            CustomerExtension(
-                FILE='DROUDI',
-                PK01=data.ROUT,
-                N096=data.PCUD if data.PCUD else None,
-                N196=data.PCUH if data.PCUH else None,
-                N296=data.PCUM if data.PCUM else None
-            )
+        for DDOW in list_of_departure_days:
+            RODN = calc_route_departure(DDOW, data.ARDY) if not data.RODN else data.RODN
+
+        yield CustomerExtension.construct(
+            FILE='DROUDI',
+            PK01=data.ROUT,
+            PK02=RODN,
+            N096=data.PCUD if data.PCUD else None,
+            N196=data.PCUH if data.PCUH else None,
+            N296=data.PCUM if data.PCUM else None
         )
 
     if data.CUSD:
-        list_of_customer_extensions.append(
-            CustomerExtension(
+        yield CustomerExtension.construct(
                 FILE='DROUTE',
                 PK01=data.ROUT
             )
-        )
 
-    return list_of_customer_extensions
-    
 
-def MakeCustomerExtensionExtended(data: Template) -> List[CustomerExtensionExtended]:
-    customer_extensions_extended_list = []
+def MakeCustomerExtensionExtended(data: ValidatedTemplate) -> Iterator[CustomerExtensionExtended]:
+    data = data.copy()
 
     if data.CUSD:
-        customer_extensions_extended_list.append(
-            CustomerExtensionExtended(
+        yield CustomerExtensionExtended.construct(
                 FILE='DROUTE',
                 PK01=data.ROUT,
                 CHB1=data.CUSD
-            )
-        )
-
-    return customer_extensions_extended_list
\ No newline at end of file
+            )
\ No newline at end of file
